/**
 * @file backend.d
 * @brief Representation of a generated D program
 * @author Sebastien Alaiwan
 * @date 2014-12-02
 */

import codestream;
import std.string;

class GProgram
{
  GStruct[] structs;
  GFunction[] functions;
}

class GStruct
{
  string name;

  GStructMember[] members;
}

class GStructMember
{
  string type;
  string name;
  int arrayLength;
  string comment;
}

class GFunction
{
  string name;
  string retType;
  string[] args;
  string[] codelines;
}

void printCode(const GProgram p, CodeStream s)
{
  foreach(gstruct; p.structs)
    printStruct(gstruct, s);

  foreach(gfunction; p.functions)
    printFunction(gfunction, s);
}

private:
void printStruct(const GStruct gstruct, CodeStream s)
{
  assert(gstruct.name != "");
  s.writefln("struct %s", gstruct.name);
  s.writefln("{");
  s.indent();

  foreach(memberDesc; gstruct.members)
    printMember(memberDesc, s);

  s.unindent();
  s.writefln("}");
  s.writefln("");
}

void printFunction(const GFunction gfunction, CodeStream s)
{
  const retType = firstNotEmpty(gfunction.retType, "void");

  s.writefln("%s %s(%s)", retType, gfunction.name,
             myJoin(gfunction.args, ", "));
  s.writefln("{");
  s.indent();

  if(gfunction.retType != "")
    s.writefln("%s r;", gfunction.retType);

  foreach(line; gfunction.codelines)
    s.writefln("%s", line);

  if(gfunction.retType != "")
    s.writefln("return r;");

  s.unindent();
  s.writefln("}");
  s.writefln("");
}

string firstNotEmpty(string a, string b)
{
  if(a != "")
    return a;
  else
    return b;
}

string myJoin(const string[] tab, string sep)
{
  string r;

  foreach(i, element; tab)
  {
    if(i > 0)
      r ~= sep;

    r ~= element;
  }

  return r;
}

void printMember(const GStructMember desc, CodeStream s)
{
  string line;
  line ~= format("%s %s", desc.type, desc.name);

  if(desc.arrayLength > 1)
    line ~= format("[%s]", desc.arrayLength);

  line ~= ";";

  if(desc.comment != "")
    line ~= format("// %s", desc.comment);

  s.writefln("%s", line);
}

