/**
 * @file pe.d
 * @author Sebastien Alaiwan
 * @date 2014-10-08
 */

/*
 * Copyright (C) 2014 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import model;

MODEL Model()
{
  return [
    mkNamedSequence
    (
      "PE",
      [
        mkField(1, "MZ_HEADER", "dos_header", 1, "DOS MZ header"),
        mkField(1, "DWORD", "padding", 1, "padding bytes"),
        mkField(1, "PE_HEADER", "pe_header", 1, ""),
        mkField(1, "PE_OPT_HEADER", "pe_opt_header", 1, ""),
      ]
    ),
    mkNamedSequence
    (
      "MZ_HEADER",
      [
        mkField(2, "WORD", "e_magic", 1, "Magic number"),
        mkField(2, "WORD", "e_cblp", 1, "Bytes on last page of file"),
        mkField(2, "WORD", "e_cp", 1, "Pages in file"),
        mkField(2, "WORD", "e_crlc", 1, "Relocations"),
        mkField(2, "WORD", "e_cparhdr", 1, "Size of header in paragraphs"),
        mkField(2, "WORD", "e_minalloc", 1, "Minimum extra paragraphs needed"),
        mkField(2, "WORD", "e_maxalloc", 1, "Maximum extra paragraphs needed"),
        mkField(2, "WORD", "e_ss", 1, "Initial (relative) SS value"),
        mkField(2, "WORD", "e_sp", 1, "Initial SP value"),
        mkField(2, "WORD", "e_csum", 1, "Checksum"),
        mkField(2, "WORD", "e_ip", 1, "Initial IP value"),
        mkField(2, "WORD", "e_cs", 1, "Initial (relative) CS value"),
        mkField(2, "WORD", "e_lfarlc", 1, "File address of relocation table"),
        mkField(2, "WORD", "e_ovno", 1, "Overlay number"),
        mkField(2, "WORD", "e_res", 4, "Reserved words"),
        mkField(2, "WORD", "e_oemid", 1, "OEM identifier (for e_oeminfo)"),
        mkField(2, "WORD", "e_oeminfo", 1, "OEM information; e_oemid specific"),
        mkField(2, "WORD", "e_res2", 10, "Reserved words"),
        mkField(4, "DWORD", "e_lfanew", 1, "File address of new exe header"),
      ]
    ),
    mkNamedSequence
    (
      "PE_HEADER",
      [
        mkField(4, "DWORD", "Magic", 1, ""),
        mkField(2, "WORD", "Machine", 1, ""),
        mkField(2, "WORD", "NumberOfSections", 1, ""),
        mkField(4, "DWORD", "TimeDateStamp", 1, ""),
        mkField(4, "DWORD", "PointerToSymbolTable", 1, ""),
        mkField(4, "DWORD", "NumberOfSymbols", 1, ""),
        mkField(2, "WORD", "SizeOfOptionalHeader", 1, ""),
        mkField(2, "WORD", "Characteristics", 1, ""),
      ]
    ),
    mkNamedSequence
    (
      "PE_OPT_HEADER",
      [
        mkField(2, "WORD", "Magic", 1, ""),
        mkField(1, "BYTE", "MajorLinkerVersion", 1, ""),
        mkField(1, "BYTE", "MinorLinkerVersion", 1, ""),
        mkField(4, "DWORD", "SizeOfCode", 1, ""),
        mkField(4, "DWORD", "SizeOfInitializedData", 1, ""),
        mkField(4, "DWORD", "SizeOfUninitializedData", 1, ""),
        mkField(4, "DWORD", "AddressOfEntryPoint", 1, ""),
        mkField(4, "DWORD", "BaseOfCode", 1, ""),
        mkField(4, "DWORD", "BaseOfData", 1, ""),
        mkField(4, "DWORD", "ImageBase", 1, ""),
        mkField(4, "DWORD", "SectionAlignment", 1, ""),
        mkField(4, "DWORD", "FileAlignment", 1, ""),
        mkField(2, "WORD", "MajorOperatingSystemVersion", 1, ""),
        mkField(2, "WORD", "MinorOperatingSystemVersion", 1, ""),
        mkField(2, "WORD", "MajorImageVersion", 1, ""),
        mkField(2, "WORD", "MinorImageVersion", 1, ""),
        mkField(2, "WORD", "MajorSubsystemVersion", 1, ""),
        mkField(2, "WORD", "MinorSubsystemVersion", 1, ""),
        mkField(4, "DWORD", "Win32VersionValue", 1, ""),
        mkField(4, "DWORD", "SizeOfImage", 1, ""),
        mkField(4, "DWORD", "SizeOfHeaders", 1, ""),
        mkField(4, "DWORD", "CheckSum", 1, ""),
        mkField(2, "WORD", "Subsystem", 1, ""),
        mkField(2, "WORD", "DllCharacteristics", 1, ""),
        mkField(4, "DWORD", "SizeOfStackReserve", 1, ""),
        mkField(4, "DWORD", "SizeOfStackCommit", 1, ""),
        mkField(4, "DWORD", "SizeOfHeapReserve", 1, ""),
        mkField(4, "DWORD", "SizeOfHeapCommit", 1, ""),
        mkField(4, "DWORD", "LoaderFlags", 1, ""),
        mkField(4, "DWORD", "NumberOfRvaAndSizes", 1, ""),
      ]
    ),
    mkNamedSequence
    (
      "PE_SECTION_HEADER",
      [
        mkField(1, "BYTE", "Name", 8, ""),
        mkField(4, "DWORD", "VirtualSize", 1, ""),
        mkField(4, "DWORD", "VirtualAddress", 1, ""),
        mkField(4, "DWORD", "SizeOfRawData", 1, ""),
        mkField(4, "DWORD", "PointerToRawData", 1, ""),
        mkField(4, "DWORD", "PointerToRelocations", 1, ""),
        mkField(4, "DWORD", "PointerToLinenumbers", 1, ""),
        mkField(2, "WORD", "NumberOfRelocations", 1, ""),
        mkField(2, "WORD", "NumberOfLinenumbers", 1, ""),
        mkField(4, "DWORD", "Characteristics", 1, ""),
      ]
    ),
    mkNamedSequence
    (
      "IMAGE_DATA_DIRECTORY",
      [
        mkField(4, "DWORD", "VirtualAddress", 1, ""),
        mkField(4, "DWORD", "Size", 1, ""),
      ]
    ),
    mkNamedSequence
    (
      "IMAGE_EXPORT_DIRECTORY",
      [
        mkField(4, "DWORD", "Characteristics", 1, ""),
        mkField(4, "DWORD", "TimeDateStamp", 1, ""),
        mkField(2, "WORD", "MajorVersion", 1, ""),
        mkField(2, "WORD", "MinorVersion", 1, ""),
        mkField(4, "DWORD", "Name", 1, ""),
        mkField(4, "DWORD", "Base", 1, ""),
        mkField(4, "DWORD", "NumberOfFunctions", 1, ""),
        mkField(4, "DWORD", "NumberOfNames", 1, ""),
        mkField(4, "DWORD", "AddressOfFunctions", 1, ""),
        mkField(4, "DWORD", "AddressOfNames", 1, ""),
        mkField(4, "DWORD", "AddressOfNameOrdinals", 1, ""),
      ]
    )
  ];
}

