import model;
import codestream;
import backend;
import std.string;

string getParserCode(MODEL m)
{
  const program = generateParser(m);

  auto s = new CodeStream;
  printCode(program, s);
  return s.toString();
}

GProgram generateParser(MODEL m)
{
  auto prog = new GProgram;

  Context ctx;

  foreach(block; m)
    generateBlock(prog, block, ctx);

  return prog;
}

private:
struct Context
{
  GStruct currStruct;
  GFunction currParseFunc;
  GFunction currPrintFunc;
}

void generateBlock(GProgram prog, in Block block, ref Context ctx)
{
  void doSequence(const(SequenceBlock)desc)
  {
    foreach(stmt; desc.statements)
      generateBlock(prog, stmt, ctx);
  }

  void doNamedSequence(const(NamedSequenceBlock)desc)
  {
    auto gstruct = new GStruct;
    gstruct.name = desc.sName;
    prog.structs ~= gstruct;
    ctx.currStruct = gstruct;

    if(ctx.currParseFunc)
    {
      ctx.currParseFunc.codelines ~= format("parse_%s(stream);", desc.sName);
    }

    {
      auto parseFunc = new GFunction;
      prog.functions ~= parseFunc;
      ctx.currParseFunc = parseFunc;

      parseFunc.retType = desc.sName;
      parseFunc.name = "parse_" ~ desc.sName;
      parseFunc.args = ["Stream stream", ];
    }

    {
      auto printFunc = new GFunction;
      prog.functions ~= printFunc;
      ctx.currPrintFunc = printFunc;

      printFunc.name = "print_" ~ desc.sName;
      printFunc.args = ["string name", "in " ~ desc.sName ~ " s", "File o"];
      printFunc.codelines ~= format(`writeln("- %s");`, desc.sName);
    }

    doSequence(desc.seq);
  }

  void doField(const FieldBlock desc)
  {
    auto member = new GStructMember;
    member.type = desc.sType;
    member.name = desc.sName;
    member.arrayLength = desc.iArrayLength;
    member.comment = desc.sComment;

    ctx.currStruct.members ~= member;

    if(desc.iArrayLength > 1)
    {
      ctx.currParseFunc.codelines ~= format("foreach(i; 0..%s)",
                                            desc.iArrayLength);
      ctx.currParseFunc.codelines ~= format("  r.%s[i] = parse_%s(stream);",
                                            desc.sName, desc.sType);
    }
    else
    {
      ctx.currParseFunc.codelines ~= format("r.%s = parse_%s(stream);",
                                            desc.sName, desc.sType);
    }

    if(desc.iArrayLength > 1)
    {
      ctx.currPrintFunc.codelines ~= format("foreach(i; 0..%s)",
                                            desc.iArrayLength);
      ctx.currPrintFunc.codelines ~= format(`  print_%s("%s", s.%s[i], o);`,
                                            desc.sType, desc.sName, desc.sName);
    }
    else
    {
      ctx.currPrintFunc.codelines ~= format(`print_%s("%s", s.%s, o);`,
                                            desc.sType, desc.sName, desc.sName);
    }
  }

  void doIf(const IfBlock ifBlock)
  {
    auto parseFunc = ctx.currParseFunc;

    parseFunc.codelines ~= "if(" ~ ifBlock.condition ~ ")";
    parseFunc.codelines ~= "{";
    ctx.currParseFunc = parseFunc;
    generateBlock(prog, ifBlock.m_then, ctx);
    parseFunc.codelines ~= "}";
    parseFunc.codelines ~= "else";
    parseFunc.codelines ~= "{";
    ctx.currParseFunc = parseFunc;
    generateBlock(prog, ifBlock.m_else, ctx);
    parseFunc.codelines ~= "}";
  }

  block.visit(
    &doNamedSequence,
    &doSequence,
    &doIf,
    &doField);
}

